package nintaco;

import java.io.*;
import nintaco.apu.*;
import nintaco.mappers.*;
import nintaco.mappers.nintendo.vs.*;
import static nintaco.mappers.nintendo.vs.VsPPU.*;

public class Machine implements Serializable {
  
  private static final long serialVersionUID = 0;

  private final Mapper mapper;
  private final CPU cpu;
  private final PPU ppu;
  private final APU apu;
  
  public Machine(final Mapper mapper, final VsGame vsGame) {
    
    this.mapper = mapper;
    
    cpu = new CPU();    
    apu = new APU();
    
    if (vsGame != null) {      
      switch(vsGame.getPPU()) {
        case RC2C05_01:
        case RC2C05_02:
        case RC2C05_03:
        case RC2C05_04:
        case RC2C05_05:
          ppu = new PPU2C05(vsGame.getPpuStatusID());
          break;
        default:
          ppu = new PPU();
          break;
      }
    } else {
      ppu = new PPU();
    }
    
    cpu.setMachine(this);
    ppu.setMachine(this);
    apu.setMachine(this);
    apu.getDMC().setCPU(cpu);
    apu.reset();
  }
  
  // For Force Ejection
  public Machine(final Mapper mapper, final Machine machine) {
    
    mapper.copyMemory(machine.getMapper());
    this.mapper = mapper;
    
    cpu = machine.getCPU();    
    apu = machine.getAPU();
    ppu = machine.getPPU();
    
    cpu.setMachine(this);
    ppu.setMachine(this);
    apu.setMachine(this);
    apu.getDMC().setCPU(cpu);
  }  

  public Mapper getMapper() {
    return mapper;
  }
  
  public CPU getCPU() {
    return cpu;
  }

  public PPU getPPU() {
    return ppu;
  }

  public APU getAPU() {
    return apu;
  }
}
