package nintaco.cartdb;

import java.io.*;
import java.util.*;
import nintaco.tv.*;
import static nintaco.tv.TVSystem.*;

public final class CartDB {
  
  private static final Map<Integer, Cart> carts = new HashMap<>();
  
  private static volatile boolean enabled = true;
  
  public static void init() {
    try(final DataInputStream in = new DataInputStream(new BufferedInputStream(
        CartDB.class.getResourceAsStream("/nintaco/cartdb/cart-db.dat")))) {
      while(true) {
        try {
          final int crc = in.readInt();        
          final int mapper = in.readUnsignedByte();
          final int submapper = in.readUnsignedByte();
          final TVSystem tvSystem = getTVSystem(in.readUnsignedByte());
          final int device = in.readByte();
          final int mirroring = in.readByte();
          carts.put(crc, new Cart(crc, mapper, submapper, tvSystem, device, 
              mirroring));
        } catch(final EOFException eof) {
          break;
        }
      }
    } catch(final Throwable t) {
      //t.printStackTrace();
    }
  }
  
  private static TVSystem getTVSystem(final int tvSystem) {
    switch(tvSystem) {
      case CartTVSystem.NTSC:
        return NTSC;
      case CartTVSystem.PAL:
        return PAL;
      default:
        return Dendy;
    }
  }
  
  public static synchronized Cart getCart(final int crc) {    
    return carts.get(crc);
  }
  
  public static synchronized void setEnabled(final boolean enabled) {
    CartDB.enabled = enabled;
  }
  
  public static synchronized boolean isEnabled() {
    return enabled;
  }
  
  private CartDB() {    
  }
}
