package nintaco.gui;

import nintaco.gui.image.preferences.Paths;
import nintaco.gui.image.ImageFrame;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import javax.swing.*;
import nintaco.files.*;
import nintaco.preferences.*;
import nintaco.mappers.nintendo.vs.*;
import static nintaco.util.BitUtil.*;
import static nintaco.files.FileUtil.*;
import static nintaco.util.MathUtil.*;
import static nintaco.util.StringUtil.*;
import static nintaco.mappers.NametableMirroring.*;
import static nintaco.tv.TVSystem.*;
import static nintaco.util.GuiUtil.*;

public class EditNesHeaderDialog extends javax.swing.JDialog {
  
  private static final String[] MEM_SIZES_16 = { "None", "16K", "32K", "64K", 
      "128K", "256K", "512K", "1M", "2M", "4M", "8M", "16M", "32M", "64M" };
  private static final String[] MEM_SIZES_8 = { "None", "8K", "16K", "32K", 
      "64K", "128K", "256K", "512K", "1M", "2M", "4M", "8M", "16M", "32M" };
  private static final String[] RAM_SIZES = { "None", "128", "256", "512", "1K", 
      "2K", "4K", "8K", "16K", "32K", "64K", "128K", "256K", "512K", "1M" };

  private final ItemListener radioButtonListener = this::radioButtonChanged;
  
  private MutableNesFile originalNesFile;
  private MutableNesFile nesFile;
  private File saveFile;
  private String entryFileName;
  
  public EditNesHeaderDialog(java.awt.Frame parent, boolean modal) {
    super(parent, modal);
    initComponents();
    getRootPane().setDefaultButton(saveAsButton);
    prgNVRamComboBox.setPrototypeDisplayValue("Disabled ");
    vsPPUComboBox.setPrototypeDisplayValue(
        VsPPU.toString(VsPPU.RP2C04_0004) + " ");
    scaleFonts(this);
    pack(); 
  }
  
  public void setEntryFileName(String entryFileName) {
    this.entryFileName = entryFileName;
  }
  
  public void setNesFile(MutableNesFile nesFile) {
    this.originalNesFile = nesFile;
    restoreAllFields();    
  }
  
  public File getSaveFile() {
    return saveFile;
  }
  
  public MutableNesFile getNesFile() {
    return nesFile;
  }
  
  private void restoreAllFields() {
    this.nesFile = (MutableNesFile)originalNesFile.copy();
    setAllFields();    
  }
  
  private void captureAllFields() {
    nesFile.setNes20Format(nes20RadioButton.isSelected());
    nesFile.setMapperNumber(clamp(parseInt(mapperTextField.getText()), 
        0, 4095));
    nesFile.setSubmapperNumber(clamp(parseInt(submapperTextField.getText()), 
        0, 15));
    if (horizontalRadioButton.isSelected()) {
      nesFile.setMirroring(HORIZONTAL);
    } else if (verticalRadioButton.isSelected()) {
      nesFile.setMirroring(VERTICAL);
    } else if (fourScreenRadioButton.isSelected()) {
      nesFile.setMirroring(FOUR_SCREEN);
    }
    if (trainerCheckBox.isSelected()) {
      if (originalNesFile.isTrainerPresent()) {
        nesFile.setTrainer(originalNesFile.getTrainer());
      } else {
        nesFile.setTrainer(new int[512]);
      }
    } else {
      nesFile.setTrainer(new int[0]);
    }
    nesFile.setAnyTVSystem(bothRadioButton.isSelected());
    nesFile.setTvSystem(palRadioButton.isSelected() ? PAL : NTSC);
    nesFile.setVsUnisystem(vsSystemRadioButton.isSelected());
    nesFile.setPlaychoice10(playChoice10RadioButton.isSelected());
    if (prgRomComboBox.getSelectedIndex() == 0) {
      nesFile.setPrgRomPages(0);
    } else {
      nesFile.setPrgRomPages(1 << (prgRomComboBox.getSelectedIndex() - 1));
    }
    if (chrRomComboBox.getSelectedIndex() == 0) {
      nesFile.setChrRomPages(0);
    } else {
      nesFile.setChrRomPages(1 << (chrRomComboBox.getSelectedIndex() - 1));
    }
    if (prgNVRamComboBox.getModel().getSize() == 2) {
      nesFile.setNonVolatilePrgRamPresent(
          prgNVRamComboBox.getSelectedIndex() == 1);
      nesFile.setNonVolatilePrgRamSize(
          nesFile.isNonVolatilePrgRamPresent() ? 0x2000 : 0);
    } else if (prgNVRamComboBox.getSelectedIndex() == 0) {
      nesFile.setNonVolatilePrgRamSize(0);
      nesFile.setNonVolatilePrgRamPresent(false);
    } else {
      nesFile.setNonVolatilePrgRamSize(
          1 << (prgNVRamComboBox.getSelectedIndex() + 6));
      nesFile.setNonVolatilePrgRamPresent(true);
    }
    if (chrNVRamComboBox.getModel().getSize() > 0) {
      if (chrNVRamComboBox.getSelectedIndex() == 0) {
        nesFile.setNonVolatileChrRamSize(0);
      } else {
        nesFile.setNonVolatileChrRamSize(
            1 << (chrNVRamComboBox.getSelectedIndex() + 6));
      }
    } 
    if (chrRamComboBox.getModel().getSize() > 0) {
      if (chrRamComboBox.getSelectedIndex() == 0) {
        nesFile.setChrRamSize(0);
      } else {
        nesFile.setChrRamSize(1 << (chrRamComboBox.getSelectedIndex() + 6));
      }
    } 
    
    if (prgRamComboBox.getSelectedIndex() == 0) {
      nesFile.setPrgRamPages(0);
      nesFile.setPrgRamSize(0);
    } else if (prgRamComboBox.getModel().getSize() == 10) {
      nesFile.setPrgRamPages(1 << (prgRamComboBox.getSelectedIndex() - 1));
      nesFile.setPrgRamSize(
          nesFile.getPrgRamPages() * NesFile.PRG_RAM_PAGE_SIZE);
    } else {
      nesFile.setPrgRamSize(1 << (prgRamComboBox.getSelectedIndex() + 6));
      nesFile.setPrgRamPages(nesFile.getPrgRamSize() 
          / NesFile.PRG_RAM_PAGE_SIZE);
    }
    
    if (vsModeComboBox.getModel().getSize() > 0) {
      nesFile.setVsMode(vsModeComboBox.getSelectedIndex());
    }
    if (vsPPUComboBox.getModel().getSize() > 0) {
      nesFile.setVsPPU(vsPPUComboBox.getSelectedIndex());
    }
  }
  
  private void setMemSizes(JComboBox comboBox, String[] romSizes, 
      int romPages, int pageSize, int origin) {
    DefaultComboBoxModel model = new DefaultComboBoxModel();
    for(int i = 0, len = nesFile.isNes20Format() ? 14 : 10; i < len; i++) {
      model.addElement(romSizes[i]);
    }
    comboBox.setModel(model);
    comboBox.setSelectedIndex(clamp(log2(ceilBase2(romPages * pageSize)) 
        - origin, 0, model.getSize() - 1)); 
  }
  
  private void setMemSizes(JComboBox comboBox, int size) {
    DefaultComboBoxModel model = new DefaultComboBoxModel(RAM_SIZES);
    comboBox.setModel(model);
    comboBox.setSelectedIndex(clamp(size == 0 ? 0 : log2(size) - 6, 0, 
        model.getSize() - 1));
  }
  
  private void setAllFields() {
    
    nes20RadioButton.removeItemListener(radioButtonListener);
    iNESRadioButton.removeItemListener(radioButtonListener);
    homeConsoleRadioButton.removeItemListener(radioButtonListener);
    vsSystemRadioButton.removeItemListener(radioButtonListener);
    playChoice10RadioButton.removeItemListener(radioButtonListener);
    
    setMemSizes(prgRomComboBox, MEM_SIZES_16, nesFile.getPrgRomPages(), 
        NesFile.PRG_ROM_PAGE_SIZE, 13);
    setMemSizes(chrRomComboBox, MEM_SIZES_8, nesFile.getChrRomPages(), 
        NesFile.CHR_ROM_PAGE_SIZE, 12);
    
    if (nesFile.isNes20Format() && nesFile.isVsUniSystem()) {
      DefaultComboBoxModel model = new DefaultComboBoxModel();
      for(int i = 0; i < 4; i++) {
        model.addElement(VsMode.toString(i));
      }
      vsModeComboBox.setModel(model);
      vsModeComboBox.setSelectedIndex(nesFile.getVsMode());
      vsModeComboBox.setEnabled(true);
      
      model = new DefaultComboBoxModel();
      for(int i = 0; i < 13; i++) {
        model.addElement(VsPPU.toString(i));
      }
      vsPPUComboBox.setModel(model);
      vsPPUComboBox.setSelectedIndex(nesFile.getVsPPU());
      vsPPUComboBox.setEnabled(true);
    } else {
      vsModeComboBox.setModel(new DefaultComboBoxModel());
      vsPPUComboBox.setModel(new DefaultComboBoxModel());
      vsModeComboBox.setEnabled(false);
      vsPPUComboBox.setEnabled(false);
    }
    
    if (nesFile.isVsUniSystem()) {
      vsSystemRadioButton.setSelected(true);
    } else if (nesFile.isPlaychoice10()) {
      playChoice10RadioButton.setSelected(true);
    } else {
      homeConsoleRadioButton.setSelected(true);
    }
    
    if (nesFile.isNes20Format()) {
      nes20RadioButton.setSelected(true);      
      submapperTextField.setText(
          Integer.toString(nesFile.getSubmapperNumber()));
      submapperTextField.setEnabled(true);
      bothRadioButton.setEnabled(true);
      if (nesFile.isAnyTVSystem()) {
        bothRadioButton.setSelected(true);        
      }
      playChoice10RadioButton.setEnabled(true);
      chrRamComboBox.setEnabled(true);
      chrNVRamComboBox.setEnabled(true);
      setMemSizes(prgNVRamComboBox, nesFile.getNonVolatilePrgRamSize());
      setMemSizes(chrRamComboBox, nesFile.getChrRamSize());
      setMemSizes(chrNVRamComboBox, nesFile.getNonVolatileChrRamSize());
      setMemSizes(prgRamComboBox, nesFile.getPrgRamSize());
    } else {            
      iNESRadioButton.setSelected(true);
      submapperTextField.setText("");
      submapperTextField.setEnabled(false);
      bothRadioButton.setEnabled(false);
      playChoice10RadioButton.setEnabled(false);
      chrRamComboBox.setEnabled(false);
      chrNVRamComboBox.setEnabled(false);
      chrRamComboBox.setModel(new DefaultComboBoxModel());
      chrNVRamComboBox.setModel(new DefaultComboBoxModel());
      DefaultComboBoxModel model = new DefaultComboBoxModel(
          new String[] { "Disabled", "Enabled" });
      prgNVRamComboBox.setModel(model);
      prgNVRamComboBox.setSelectedIndex(
          nesFile.isNonVolatilePrgRamPresent() ? 1 : 0);
      setMemSizes(prgRamComboBox, MEM_SIZES_8, nesFile.getPrgRamPages(), 
          NesFile.PRG_RAM_PAGE_SIZE, 12);
    }
    mapperTextField.setText(Integer.toString(nesFile.getMapperNumber()));
    switch(nesFile.getMirroring()) {
      case HORIZONTAL:
        horizontalRadioButton.setSelected(true);
        break;
      case VERTICAL:
        verticalRadioButton.setSelected(true);
        break;
      case FOUR_SCREEN:
        fourScreenRadioButton.setSelected(true);
        break;
    }
    trainerCheckBox.setSelected(nesFile.isTrainerPresent());  
    if (!(nesFile.isNes20Format() && nesFile.isAnyTVSystem())) {
      if (nesFile.getTvSystem() == NTSC) {
        ntscRadioButton.setSelected(true);
      } else {
        palRadioButton.setSelected(true);
      }
    }
    
    nes20RadioButton.addItemListener(radioButtonListener);
    iNESRadioButton.addItemListener(radioButtonListener);  
    homeConsoleRadioButton.addItemListener(radioButtonListener);
    vsSystemRadioButton.addItemListener(radioButtonListener);
    playChoice10RadioButton.addItemListener(radioButtonListener);
  }
  
  private void radioButtonChanged(ItemEvent e) {
    if (e.getStateChange() == ItemEvent.SELECTED) {
      EventQueue.invokeLater(() -> {
        captureAllFields();
        setAllFields();
      });
    }
  }
  
  private void closeDialog() {    
    dispose();
  }

  /**
   * This method is called from within the constructor to initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is always
   * regenerated by the Form Editor.
   */
  @SuppressWarnings("unchecked")
  // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {

    fileFormatButtonGroup = new javax.swing.ButtonGroup();
    mirroringButtonGroup = new javax.swing.ButtonGroup();
    tvSystemButtonGroup = new javax.swing.ButtonGroup();
    machineButtonGroup = new javax.swing.ButtonGroup();
    leftPanel = new javax.swing.JPanel();
    mirroringLabel = new javax.swing.JLabel();
    fourScreenRadioButton = new javax.swing.JRadioButton();
    iNESRadioButton = new javax.swing.JRadioButton();
    fileFormatLabel = new javax.swing.JLabel();
    nes20RadioButton = new javax.swing.JRadioButton();
    horizontalRadioButton = new javax.swing.JRadioButton();
    mapperTextField = new javax.swing.JTextField();
    submapperLabel = new javax.swing.JLabel();
    verticalRadioButton = new javax.swing.JRadioButton();
    mapperLabel = new javax.swing.JLabel();
    submapperTextField = new javax.swing.JTextField();
    middlePanel = new javax.swing.JPanel();
    prgRomLabel = new javax.swing.JLabel();
    prgRomComboBox = new javax.swing.JComboBox();
    prgRamLabel = new javax.swing.JLabel();
    prgRamComboBox = new javax.swing.JComboBox();
    prgNVRamLabel = new javax.swing.JLabel();
    prgNVRamComboBox = new javax.swing.JComboBox();
    chrRomLabel = new javax.swing.JLabel();
    chrRamLabel = new javax.swing.JLabel();
    chrNVRamLabel = new javax.swing.JLabel();
    chrNVRamComboBox = new javax.swing.JComboBox();
    chrRamComboBox = new javax.swing.JComboBox();
    chrRomComboBox = new javax.swing.JComboBox();
    trainerCheckBox = new javax.swing.JCheckBox();
    bottomPanel = new javax.swing.JPanel();
    CancelButton = new javax.swing.JButton();
    RestoreAllButton = new javax.swing.JButton();
    saveAsButton = new javax.swing.JButton();
    rightPanel = new javax.swing.JPanel();
    MachineLabel = new javax.swing.JLabel();
    homeConsoleRadioButton = new javax.swing.JRadioButton();
    vsSystemRadioButton = new javax.swing.JRadioButton();
    playChoice10RadioButton = new javax.swing.JRadioButton();
    vsPPULabel = new javax.swing.JLabel();
    vsPPUComboBox = new javax.swing.JComboBox();
    vsModeLabel = new javax.swing.JLabel();
    vsModeComboBox = new javax.swing.JComboBox();
    tvSystemLabel = new javax.swing.JLabel();
    ntscRadioButton = new javax.swing.JRadioButton();
    palRadioButton = new javax.swing.JRadioButton();
    bothRadioButton = new javax.swing.JRadioButton();

    setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
    setTitle("iNES File Header Editor");
    setPreferredSize(null);
    addWindowListener(new java.awt.event.WindowAdapter() {
      public void windowClosing(java.awt.event.WindowEvent evt) {
        formWindowClosing(evt);
      }
    });

    mirroringLabel.setText("Mirroring:");
    mirroringLabel.setPreferredSize(null);

    mirroringButtonGroup.add(fourScreenRadioButton);
    fourScreenRadioButton.setText("Four-screen");
    fourScreenRadioButton.setFocusPainted(false);

    fileFormatButtonGroup.add(iNESRadioButton);
    iNESRadioButton.setSelected(true);
    iNESRadioButton.setText("iNES");
    iNESRadioButton.setFocusPainted(false);

    fileFormatLabel.setText("File Format:");
    fileFormatLabel.setPreferredSize(null);

    fileFormatButtonGroup.add(nes20RadioButton);
    nes20RadioButton.setText("NES 2.0");
    nes20RadioButton.setFocusPainted(false);

    mirroringButtonGroup.add(horizontalRadioButton);
    horizontalRadioButton.setSelected(true);
    horizontalRadioButton.setText("Horizontal");
    horizontalRadioButton.setFocusPainted(false);

    mapperTextField.setColumns(4);
    mapperTextField.setPreferredSize(null);

    submapperLabel.setText("Submapper:");
    submapperLabel.setPreferredSize(null);

    mirroringButtonGroup.add(verticalRadioButton);
    verticalRadioButton.setText("Vertical");
    verticalRadioButton.setFocusPainted(false);

    mapperLabel.setText("Mapper:");
    mapperLabel.setPreferredSize(null);

    submapperTextField.setColumns(4);
    submapperTextField.setPreferredSize(null);

    javax.swing.GroupLayout leftPanelLayout = new javax.swing.GroupLayout(leftPanel);
    leftPanel.setLayout(leftPanelLayout);
    leftPanelLayout.setHorizontalGroup(
      leftPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(leftPanelLayout.createSequentialGroup()
        .addContainerGap()
        .addGroup(leftPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
          .addComponent(mapperLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(submapperLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(mirroringLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(fileFormatLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
        .addGroup(leftPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
          .addComponent(iNESRadioButton)
          .addComponent(nes20RadioButton)
          .addComponent(mapperTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 40, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(submapperTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(horizontalRadioButton)
          .addComponent(verticalRadioButton)
          .addComponent(fourScreenRadioButton))
        .addGap(10, 10, 10))
    );

    leftPanelLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {mapperTextField, submapperTextField});

    leftPanelLayout.setVerticalGroup(
      leftPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(leftPanelLayout.createSequentialGroup()
        .addContainerGap()
        .addGroup(leftPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(fileFormatLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(iNESRadioButton))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(nes20RadioButton)
        .addGap(29, 29, 29)
        .addGroup(leftPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(mapperLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(mapperTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addGroup(leftPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
          .addComponent(submapperLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(submapperTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        .addGap(34, 34, 34)
        .addGroup(leftPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(mirroringLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(horizontalRadioButton))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(verticalRadioButton)
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(fourScreenRadioButton)
        .addContainerGap())
    );

    prgRomLabel.setText("PRG ROM:");
    prgRomLabel.setPreferredSize(null);

    prgRomComboBox.setFocusable(false);
    prgRomComboBox.setPreferredSize(null);

    prgRamLabel.setText("PRG RAM:");
    prgRamLabel.setPreferredSize(null);

    prgRamComboBox.setFocusable(false);
    prgRamComboBox.setPreferredSize(null);

    prgNVRamLabel.setText("PRG NVRAM:");
    prgNVRamLabel.setPreferredSize(null);

    prgNVRamComboBox.setFocusable(false);
    prgNVRamComboBox.setPreferredSize(null);

    chrRomLabel.setText("CHR ROM:");
    chrRomLabel.setPreferredSize(null);

    chrRamLabel.setText("CHR RAM:");
    chrRamLabel.setPreferredSize(null);

    chrNVRamLabel.setText("CHR NVRAM:");
    chrNVRamLabel.setPreferredSize(null);

    chrNVRamComboBox.setFocusable(false);
    chrNVRamComboBox.setPreferredSize(null);

    chrRamComboBox.setFocusable(false);
    chrRamComboBox.setPreferredSize(null);

    chrRomComboBox.setFocusable(false);
    chrRomComboBox.setPreferredSize(null);

    trainerCheckBox.setText("Trainer");
    trainerCheckBox.setFocusPainted(false);

    javax.swing.GroupLayout middlePanelLayout = new javax.swing.GroupLayout(middlePanel);
    middlePanel.setLayout(middlePanelLayout);
    middlePanelLayout.setHorizontalGroup(
      middlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, middlePanelLayout.createSequentialGroup()
        .addContainerGap()
        .addGroup(middlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
          .addComponent(trainerCheckBox)
          .addGroup(middlePanelLayout.createSequentialGroup()
            .addGroup(middlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
              .addComponent(prgRomLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
              .addComponent(prgRamLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
              .addComponent(prgNVRamLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
              .addComponent(chrRomLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
              .addComponent(chrRamLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
              .addComponent(chrNVRamLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
            .addGroup(middlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
              .addComponent(prgNVRamComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
              .addComponent(prgRamComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
              .addComponent(prgRomComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
              .addComponent(chrRomComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
              .addComponent(chrRamComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
              .addComponent(chrNVRamComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
        .addContainerGap())
    );
    middlePanelLayout.setVerticalGroup(
      middlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, middlePanelLayout.createSequentialGroup()
        .addContainerGap()
        .addComponent(trainerCheckBox)
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        .addGroup(middlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(prgRomLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(prgRomComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addGroup(middlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(prgRamComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(prgRamLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addGroup(middlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(prgNVRamComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(prgNVRamLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        .addGap(18, 18, 18)
        .addGroup(middlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(chrRomLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(chrRomComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addGroup(middlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(chrRamComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(chrRamLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addGroup(middlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(chrNVRamComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(chrNVRamLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        .addContainerGap())
    );

    CancelButton.setMnemonic('C');
    CancelButton.setText("Cancel");
    CancelButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        CancelButtonActionPerformed(evt);
      }
    });

    RestoreAllButton.setMnemonic('R');
    RestoreAllButton.setText("Restore All");
    RestoreAllButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        RestoreAllButtonActionPerformed(evt);
      }
    });

    saveAsButton.setMnemonic('S');
    saveAsButton.setText("Save As...");
    saveAsButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        saveAsButtonActionPerformed(evt);
      }
    });

    javax.swing.GroupLayout bottomPanelLayout = new javax.swing.GroupLayout(bottomPanel);
    bottomPanel.setLayout(bottomPanelLayout);
    bottomPanelLayout.setHorizontalGroup(
      bottomPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, bottomPanelLayout.createSequentialGroup()
        .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        .addComponent(saveAsButton)
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(RestoreAllButton)
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(CancelButton)
        .addGap(0, 0, 0))
    );

    bottomPanelLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {CancelButton, RestoreAllButton, saveAsButton});

    bottomPanelLayout.setVerticalGroup(
      bottomPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, bottomPanelLayout.createSequentialGroup()
        .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        .addGroup(bottomPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(CancelButton)
          .addComponent(RestoreAllButton)
          .addComponent(saveAsButton))
        .addContainerGap())
    );

    MachineLabel.setText("Machine:");
    MachineLabel.setPreferredSize(null);

    machineButtonGroup.add(homeConsoleRadioButton);
    homeConsoleRadioButton.setSelected(true);
    homeConsoleRadioButton.setText("Home console");
    homeConsoleRadioButton.setFocusPainted(false);

    machineButtonGroup.add(vsSystemRadioButton);
    vsSystemRadioButton.setText("VS System");
    vsSystemRadioButton.setFocusPainted(false);

    machineButtonGroup.add(playChoice10RadioButton);
    playChoice10RadioButton.setText("PlayChoice-10");
    playChoice10RadioButton.setFocusPainted(false);

    vsPPULabel.setText("VS PPU:");
    vsPPULabel.setPreferredSize(null);

    vsPPUComboBox.setFocusable(false);
    vsPPUComboBox.setPreferredSize(null);

    vsModeLabel.setText("VS Mode:");
    vsModeLabel.setPreferredSize(null);

    vsModeComboBox.setFocusable(false);
    vsModeComboBox.setPreferredSize(null);

    tvSystemLabel.setText("TV System:");
    tvSystemLabel.setPreferredSize(null);

    tvSystemButtonGroup.add(ntscRadioButton);
    ntscRadioButton.setSelected(true);
    ntscRadioButton.setText("NTSC");
    ntscRadioButton.setFocusPainted(false);

    tvSystemButtonGroup.add(palRadioButton);
    palRadioButton.setText("PAL");
    palRadioButton.setFocusPainted(false);

    tvSystemButtonGroup.add(bothRadioButton);
    bothRadioButton.setText("Both");
    bothRadioButton.setFocusPainted(false);

    javax.swing.GroupLayout rightPanelLayout = new javax.swing.GroupLayout(rightPanel);
    rightPanel.setLayout(rightPanelLayout);
    rightPanelLayout.setHorizontalGroup(
      rightPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(rightPanelLayout.createSequentialGroup()
        .addContainerGap()
        .addGroup(rightPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
          .addComponent(MachineLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(tvSystemLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(vsPPULabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(vsModeLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
        .addGroup(rightPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
          .addComponent(vsPPUComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
          .addComponent(ntscRadioButton)
          .addComponent(palRadioButton)
          .addComponent(bothRadioButton)
          .addComponent(homeConsoleRadioButton)
          .addComponent(vsSystemRadioButton)
          .addComponent(playChoice10RadioButton)
          .addComponent(vsModeComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        .addContainerGap())
    );
    rightPanelLayout.setVerticalGroup(
      rightPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, rightPanelLayout.createSequentialGroup()
        .addContainerGap()
        .addGroup(rightPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(tvSystemLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(ntscRadioButton))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(palRadioButton)
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(bothRadioButton)
        .addGap(18, 18, 18)
        .addGroup(rightPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(MachineLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(homeConsoleRadioButton))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(vsSystemRadioButton)
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(playChoice10RadioButton)
        .addGap(18, 18, 18)
        .addGroup(rightPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(vsPPULabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(vsPPUComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addGroup(rightPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(vsModeLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
          .addComponent(vsModeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
    );

    javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
    getContentPane().setLayout(layout);
    layout.setHorizontalGroup(
      layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(layout.createSequentialGroup()
        .addComponent(bottomPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        .addGap(10, 10, 10))
      .addGroup(layout.createSequentialGroup()
        .addComponent(leftPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(middlePanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(rightPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
    );
    layout.setVerticalGroup(
      layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(layout.createSequentialGroup()
        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
          .addComponent(leftPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
          .addComponent(rightPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
          .addComponent(middlePanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(bottomPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
    );
  }// </editor-fold>//GEN-END:initComponents

  private void RestoreAllButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_RestoreAllButtonActionPerformed
    restoreAllFields();
  }//GEN-LAST:event_RestoreAllButtonActionPerformed

  private void CancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_CancelButtonActionPerformed
    closeDialog();
  }//GEN-LAST:event_CancelButtonActionPerformed

  private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
    closeDialog();
  }//GEN-LAST:event_formWindowClosing

  private void saveAsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveAsButtonActionPerformed
    final Paths paths = AppPrefs.getInstance().getPaths();    
    final File file = showSaveAsDialog(this, 
        paths.getSaveEditedNesFileDir(), entryFileName, "nes", 
            ImageFrame.FileExtensionFilters[1], true);
    if (file != null) {
      captureAllFields();
      paths.setSaveEditedNesFileDir(file.getParentFile().getPath());
      AppPrefs.save();
      saveFile = file;
      closeDialog();
    }    
  }//GEN-LAST:event_saveAsButtonActionPerformed

  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JButton CancelButton;
  private javax.swing.JLabel MachineLabel;
  private javax.swing.JButton RestoreAllButton;
  private javax.swing.JRadioButton bothRadioButton;
  private javax.swing.JPanel bottomPanel;
  private javax.swing.JComboBox chrNVRamComboBox;
  private javax.swing.JLabel chrNVRamLabel;
  private javax.swing.JComboBox chrRamComboBox;
  private javax.swing.JLabel chrRamLabel;
  private javax.swing.JComboBox chrRomComboBox;
  private javax.swing.JLabel chrRomLabel;
  private javax.swing.ButtonGroup fileFormatButtonGroup;
  private javax.swing.JLabel fileFormatLabel;
  private javax.swing.JRadioButton fourScreenRadioButton;
  private javax.swing.JRadioButton homeConsoleRadioButton;
  private javax.swing.JRadioButton horizontalRadioButton;
  private javax.swing.JRadioButton iNESRadioButton;
  private javax.swing.JPanel leftPanel;
  private javax.swing.ButtonGroup machineButtonGroup;
  private javax.swing.JLabel mapperLabel;
  private javax.swing.JTextField mapperTextField;
  private javax.swing.JPanel middlePanel;
  private javax.swing.ButtonGroup mirroringButtonGroup;
  private javax.swing.JLabel mirroringLabel;
  private javax.swing.JRadioButton nes20RadioButton;
  private javax.swing.JRadioButton ntscRadioButton;
  private javax.swing.JRadioButton palRadioButton;
  private javax.swing.JRadioButton playChoice10RadioButton;
  private javax.swing.JComboBox prgNVRamComboBox;
  private javax.swing.JLabel prgNVRamLabel;
  private javax.swing.JComboBox prgRamComboBox;
  private javax.swing.JLabel prgRamLabel;
  private javax.swing.JComboBox prgRomComboBox;
  private javax.swing.JLabel prgRomLabel;
  private javax.swing.JPanel rightPanel;
  private javax.swing.JButton saveAsButton;
  private javax.swing.JLabel submapperLabel;
  private javax.swing.JTextField submapperTextField;
  private javax.swing.JCheckBox trainerCheckBox;
  private javax.swing.ButtonGroup tvSystemButtonGroup;
  private javax.swing.JLabel tvSystemLabel;
  private javax.swing.JRadioButton verticalRadioButton;
  private javax.swing.JComboBox vsModeComboBox;
  private javax.swing.JLabel vsModeLabel;
  private javax.swing.JComboBox vsPPUComboBox;
  private javax.swing.JLabel vsPPULabel;
  private javax.swing.JRadioButton vsSystemRadioButton;
  // End of variables declaration//GEN-END:variables
}
