package nintaco.mappers.nintendo.vs;

import java.util.*;
import nintaco.files.*;
import nintaco.input.dipswitches.*;
import static nintaco.mappers.NametableMirroring.*;
import static nintaco.mappers.nintendo.vs.VsMode.*;
import static nintaco.mappers.nintendo.vs.VsPPU.*;

public enum VsGame {
    
  // unsupported VS. DualSystem games
  BalloonFight(0, FOUR_SCREEN, RP2C03B, Normal, false, false),
  Baseball(99, FOUR_SCREEN, RP2C04_0002, Normal, false, false),  
  Mahjong(0, FOUR_SCREEN, RP2C03B, Normal, false, false),
  Tennis(0, FOUR_SCREEN, RP2C03B, Normal, false, false),
  WreckingCrew(0, FOUR_SCREEN, RP2C04_0002, Normal, false, false),
  
  // supported VS. UniSystem games
  BattleCity(99, FOUR_SCREEN, RP2C04_0001, Normal, false, false),
  Castlevania(2, VERTICAL, RP2C04_0002, Normal, false, false),
  CluCluLand(99, FOUR_SCREEN, RP2C04_0004, Normal, true, false),
  DrMario(1, HORIZONTAL, RP2C04_0003, Normal, true, false),
  DuckHunt(99, FOUR_SCREEN, RP2C03B, Normal, false, true),
  Excitebike(99, FOUR_SCREEN, RP2C04_0003, Normal, false, false),
  ExcitebikeJapan(99, FOUR_SCREEN, RP2C04_0004, Normal, false, false),
  FreedomForce(4, VERTICAL, RP2C04_0001, Normal, false, true),
  Goonies(75, FOUR_SCREEN, RP2C04_0003, Normal, false, false),
  Gradius(75, FOUR_SCREEN, RP2C04_0001, Normal, true, false),
  Gumshoe(99, FOUR_SCREEN, RC2C05_03, Normal, false, true, 0x1C),
  HogansAlley(99, FOUR_SCREEN, RP2C04_0001, Normal, false, true),
  IceClimber(99, FOUR_SCREEN, RP2C04_0004, Normal, true, false),
  LadiesGolf(99, FOUR_SCREEN, RP2C04_0002, Normal, true, false),
  MachRider(99, FOUR_SCREEN, RP2C04_0002, Normal, false, false),
  MachRiderJapan(99, FOUR_SCREEN, RP2C04_0001, Normal, false, false),  
  MightyBombJack(0, FOUR_SCREEN, RC2C05_02, Normal, false, false, 0x3D),
  NinjaJajamaruKun(99, FOUR_SCREEN, RC2C05_01, Normal, true, false),
  Pinball(99, FOUR_SCREEN, RP2C04_0001, Normal, false, false),
  PinballJapan(99, FOUR_SCREEN, RP2C03B, Normal, false, false),
  Platoon(68, VERTICAL, RP2C04_0001, Normal, false, false),
  RbiBaseball(206, VERTICAL, RP2C04_0002, VsMode.RbiBaseball, true, false),
  Slalom(0, FOUR_SCREEN, RP2C04_0002, Normal, false, false),
  Soccer(99, FOUR_SCREEN, RP2C04_0003, Normal, true, false),
  StarLuster(99, FOUR_SCREEN, RP2C03B, Normal, false, false),
  StrokeAndMatchGolf(99, FOUR_SCREEN, RP2C04_0002, Normal, true, false),
  StrokeAndMatchGolfJapan(99, FOUR_SCREEN, RP2C03B, Normal, true, false),
  SuperMarioBros(99, FOUR_SCREEN, RP2C04_0004, Normal, false, false),
  SuperSkyKid(206, VERTICAL, RP2C04_0002, Normal, true, false),
  SuperXevious(206, HORIZONTAL, RP2C04_0002, VsMode.SuperXevious, false, false),  
  Tetris(99, FOUR_SCREEN, RP2C04_0002, Normal, true, false),
  TkoBoxing(206, VERTICAL, RP2C03B, VsMode.TkoBoxing, false, false),
  TopGun(2, HORIZONTAL, RC2C05_04, Normal, false, false, 0x1B);
  
  static {
    
    BattleCity.add("Credits for 2 Players", 1)
        .add("1", 0x00)
        .add("2", 0x01);
    BattleCity.add("Lives")
        .add("3", 0x00)
        .add("5", 0x02);
    BattleCity.add("Demo Sounds", 1)
        .add("Off", 0x00)
        .add("On", 0x04);
    BattleCity.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x08);
    BattleCity.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x10);
    BattleCity.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x20);
    BattleCity.add("PPU")
        .add("RP2C04-0001", 0x00)
        .add("RP2C04-0002", 0x40)
        .add("RP2C04-0003", 0x80)
        .add("RP2C04-0004", 0xC0);    
    
    Castlevania.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("2 Coins / 1 Credit", 0x06)
        .add("3 Coins / 1 Credit", 0x01)
        .add("4 Coins / 1 Credit", 0x05)
        .add("5 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    Castlevania.add("Lives", 1)
        .add("2", 0x08)
        .add("3", 0x00);
    Castlevania.add("Bonus")
        .add("100k", 0x00)
        .add("200k", 0x20)
        .add("300k", 0x10)
        .add("400k", 0x30);
    Castlevania.add("Difficulty")
        .add("Normal", 0x00)
        .add("Hard", 0x40);
    
    CluCluLand.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("1 Coin / 4 Credits", 0x06)
        .add("2 Coins / 1 Credit", 0x01)
        .add("3 Coins / 1 Credit", 0x05)
        .add("4 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    CluCluLand.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x08);
    CluCluLand.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x10);
    CluCluLand.add("Lives", 1)
        .add("2", 0x60)
        .add("3", 0x00)
        .add("4", 0x40)
        .add("5", 0x20);
    CluCluLand.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x80);    
    
    DrMario.add("Drop Rate Increases After")
        .add("7 Pills", 0x00)
        .add("8 Pills", 0x01)
        .add("9 Pills", 0x02)
        .add("10 Pills", 0x03);
    DrMario.add("Virus Level")
        .add("1", 0x00)
        .add("3", 0x04)
        .add("5", 0x08)
        .add("7", 0x0C);
    DrMario.add("Drop Speed Up")
        .add("Slow", 0x00)
        .add("Medium", 0x10)
        .add("Fast", 0x20)
        .add("Fastest", 0x30);
    DrMario.add("Free Play")
        .add("Off", 0x00)
        .add("On", 0x40);
    DrMario.add("Demo Sounds", 1)
        .add("Off", 0x00)
        .add("On", 0x80);    
    
    DuckHunt.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("2 Coins / 1 Credit", 0x06)
        .add("3 Coins / 1 Credit", 0x01)
        .add("4 Coins / 1 Credit", 0x05)
        .add("5 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    DuckHunt.add("Difficulty", 1)
        .add("Easy", 0x00)
        .add("Normal", 0x08)
        .add("Hard", 0x10)
        .add("Very Hard", 0x18);
    DuckHunt.add("Misses per Game", 1)
        .add("3", 0x00)
        .add("5", 0x20);
    DuckHunt.add("Bonus Life")
        .add("30000", 0x00)
        .add("50000", 0x40)
        .add("80000", 0x80)
        .add("100000", 0xC0);
    
    Excitebike.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("1 Coin / 4 Credits", 0x06)
        .add("2 Coins / 1 Credit", 0x01)
        .add("3 Coins / 1 Credit", 0x05)
        .add("4 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    Excitebike.add("Bonus")
        .add("100k and Every 50k", 0x00)
        .add("Every 100k", 0x10)
        .add("100k Only", 0x08)
        .add("None", 0x18);
    Excitebike.add("1st Half Qualifying Time")
        .add("Normal", 0x00)
        .add("Hard", 0x20);
    Excitebike.add("2nd Half Qualifying Time")
        .add("Normal", 0x00)
        .add("Hard", 0x40); 
    
    ExcitebikeJapan.addAll(Excitebike.getDipSwitches());
    
    FreedomForce.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("2 Coins / 1 Credit", 0x06)
        .add("3 Coins / 1 Credit", 0x01)
        .add("4 Coins / 1 Credit", 0x05)
        .add("5 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    FreedomForce.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x08);
    FreedomForce.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x10);
    FreedomForce.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x20);
    FreedomForce.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x40);
    FreedomForce.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x80);
    
    LadiesGolf.add("Coinage")
        .add("1 Coin / 1 Credit", 0x01)
        .add("1 Coin / 2 Credits", 0x06)
        .add("1 Coin / 3 Credits", 0x02)
        .add("1 Coin / 4 Credits", 0x04)
        .add("2 Coins / 1 Credit", 0x05)
        .add("3 Coins / 1 Credit", 0x03)
        .add("4 Coins / 1 Credit", 0x07)
        .add("Free Play", 0x00);
    LadiesGolf.add("Hole Size")
        .add("Large", 0x00)
        .add("Small", 0x08);
    LadiesGolf.add("Points per Stroke")
        .add("Easier", 0x00)
        .add("Harder", 0x10);
    LadiesGolf.add("Starting Points")
        .add("10", 0x00)
        .add("13", 0x40)
        .add("16", 0x20)
        .add("20", 0x60);
    LadiesGolf.add("Difficulty Vs. Computer")
        .add("Easy", 0x00)
        .add("Hard", 0x80); 
    
    Goonies.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("2 Coins / 1 Credit", 0x06)
        .add("3 Coins / 1 Credit", 0x01)
        .add("4 Coins / 1 Credit", 0x05)
        .add("5 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    Goonies.add("Lives")
        .add("3", 0x00)
        .add("2", 0x08);
    Goonies.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x10);
    Goonies.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x20);
    Goonies.add("Timer")
        .add("Normal", 0x00)
        .add("Fast", 0x40);
    Goonies.add("Demo Sounds", 1)
        .add("Off", 0x00)
        .add("On", 0x80);    
    
    Gradius.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("2 Coins / 1 Credit", 0x06)
        .add("3 Coins / 1 Credit", 0x01)
        .add("4 Coins / 1 Credit", 0x05)
        .add("5 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    Gradius.add("Lives")
        .add("3", 0x08)
        .add("4", 0x00);
    Gradius.add("Bonus")
        .add("100k", 0x00)
        .add("200k", 0x20)
        .add("300k", 0x10)
        .add("400k", 0x30);
    Gradius.add("Difficulty")
        .add("Normal", 0x00)
        .add("Hard", 0x40);
    Gradius.add("Demo Sounds", 1)
        .add("Off", 0x00)
        .add("On", 0x80); 
    
    Gumshoe.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("2 Coins / 1 Credit", 0x06)
        .add("3 Coins / 1 Credit", 0x01)
        .add("4 Coins / 1 Credit", 0x05)
        .add("5 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    Gumshoe.add("Difficulty", 1)
        .add("Easy", 0x00)
        .add("Normal", 0x08)
        .add("Hard", 0x10)
        .add("Very Hard", 0x18);
    Gumshoe.add("Lives", 1)
        .add("3", 0x10)
        .add("5", 0x00);
    Gumshoe.add("Bullets per Balloon", 1)
        .add("2", 0x40)
        .add("3", 0x00);
    Gumshoe.add("Bonus Life")
        .add("80000", 0x00)
        .add("100000", 0x80);  
    
    HogansAlley.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("2 Coins / 1 Credit", 0x06)
        .add("3 Coins / 1 Credit", 0x01)
        .add("4 Coins / 1 Credit", 0x05)
        .add("5 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    HogansAlley.add("Difficulty", 1)
        .add("Easy", 0x00)
        .add("Normal", 0x08)
        .add("Hard", 0x10)
        .add("Very Hard", 0x18);
    HogansAlley.add("Misses per Game", 1)
        .add("3", 0x00)
        .add("5", 0x20);
    HogansAlley.add("Bonus Life")
        .add("30000", 0x00)
        .add("50000", 0x40)
        .add("80000", 0x80)
        .add("100000", 0xC0);    
    
    IceClimber.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("1 Coin / 4 Credits", 0x06)
        .add("2 Coins / 1 Credit", 0x01)
        .add("3 Coins / 1 Credit", 0x05)
        .add("4 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    IceClimber.add("Lives")
        .add("3", 0x00)
        .add("4", 0x10)
        .add("5", 0x08)
        .add("7", 0x18);
    IceClimber.add("Difficulty")
        .add("Normal", 0x00)
        .add("Hard", 0x20);
    IceClimber.add("Time before the bear")
        .add("Long", 0x00)
        .add("Short", 0x40);
    
    MachRider.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("1 Coin / 4 Credits", 0x06)
        .add("2 Coins / 1 Credit", 0x01)
        .add("3 Coins / 1 Credit", 0x05)
        .add("4 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    MachRider.add("Time")
        .add("280", 0x00)
        .add("250", 0x10)
        .add("220", 0x08)
        .add("200", 0x18);
    MachRider.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x20);
    MachRider.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x40);
    MachRider.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x80);  
    
    MachRiderJapan.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("1 Coin / 4 Credits", 0x06)
        .add("2 Coins / 1 Credit", 0x01)
        .add("3 Coins / 1 Credit", 0x05)
        .add("4 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    MachRiderJapan.add("Km 1st Race")
        .add("12", 0x00)
        .add("15", 0x10);
    MachRiderJapan.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x20);
    MachRiderJapan.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x40);
    MachRiderJapan.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x80);
    
    MightyBombJack.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("1 Coin / 4 Credits", 0x06)
        .add("2 Coins / 1 Credit", 0x01)
        .add("3 Coins / 1 Credit", 0x05)
        .add("4 Coins / 1 Credit", 0x03)
        .add("5 Coins / 1 Credit", 0x07);
    MightyBombJack.add("Lives")
        .add("2", 0x10)
        .add("3", 0x00)
        .add("4", 0x08)
        .add("5", 0x18);
    MightyBombJack.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x20);
    MightyBombJack.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x40);
    MightyBombJack.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x80);   
    
    NinjaJajamaruKun.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("1 Coin / 4 Credits", 0x06)
        .add("2 Coins / 1 Credit", 0x01)
        .add("3 Coins / 1 Credit", 0x05)
        .add("4 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    NinjaJajamaruKun.add("Lives")
        .add("3", 0x00)
        .add("4", 0x10)
        .add("5", 0x08);
    NinjaJajamaruKun.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x20);
    NinjaJajamaruKun.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x40);
    NinjaJajamaruKun.add("Demo Sounds", 1)
        .add("Off", 0x00)
        .add("On", 0x80);
    
    Pinball.add("Coinage")
        .add("1 Coin / 1 Credit", 0x01)
        .add("1 Coin / 2 Credits", 0x06)
        .add("1 Coin / 3 Credits", 0x02)
        .add("1 Coin / 4 Credits", 0x04)
        .add("2 Coins / 1 Credit", 0x05)
        .add("3 Coins / 1 Credit", 0x03)
        .add("4 Coins / 1 Credit", 0x07)
        .add("Free Play", 0x00);
    Pinball.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x08);
    Pinball.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x10);
    Pinball.add("Balls", 1)
        .add("2", 0x60)
        .add("3", 0x00)
        .add("4", 0x40)
        .add("5", 0x20);
    Pinball.add("Ball Speed")
        .add("Normal", 0x00)
        .add("Fast", 0x80); 
    
    PinballJapan.addAll(Pinball.getDipSwitches());
    
    Platoon.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x01);
    Platoon.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x02);
    Platoon.add("Demo Sounds", 1)
        .add("Off", 0x00)
        .add("On", 0x04);
    Platoon.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x08);
    Platoon.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x10);
    Platoon.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x20)
        .add("1 Coin / 3 Credits", 0x40)
        .add("2 Coins / 1 Credit", 0x60)
        .add("3 Coins / 1 Credit", 0x80)
        .add("4 Coins / 1 Credit", 0xA0)
        .add("5 Coins / 1 Credit", 0xC0)
        .add("Free Play", 0xE0);   
    
    // Raid on Bungeling Bay not supported.
    
    RbiBaseball.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x01)
        .add("2 Coins / 1 Credit", 0x02)
        .add("3 Coins / 1 Credit", 0x03);
    RbiBaseball.add("Max. 1p/in, 2p/in, Min", 1)
        .add("2, 1, 3", 0x04)
        .add("2, 2, 4", 0x0C)
        .add("3, 2, 6", 0x00)
        .add("4, 3, 7", 0x08);
    RbiBaseball.add("Demo Sounds", 1)
        .add("Off", 0x00)
        .add("On", 0x10);
    RbiBaseball.add("PPU", 2)
        .add("RP2C03", 0x20)
        .add("RP2C04-0001", 0x00)
        .add("RP2C04-0002", 0x40)
        .add("RP2C04-0003", 0x80)
        .add("RP2C04-0004", 0xC0);
    
    Slalom.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("2 Coins / 1 Credit", 0x06)
        .add("3 Coins / 1 Credit", 0x01)
        .add("4 Coins / 1 Credit", 0x05)
        .add("5 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    Slalom.add("Freestyle Points")
        .add("Left / Right", 0x00)
        .add("Hold Time", 0x08);
    Slalom.add("Difficulty", 1)
        .add("Easy", 0x00)
        .add("Normal", 0x10)
        .add("Hard", 0x20)
        .add("Hardest", 0x30);
    Slalom.add("Allow Continue", 1)
        .add("No", 0x40)
        .add("Yes", 0x00);
    Slalom.add("Inverted input")
        .add("Off", 0x00)
        .add("On", 0x80);
    
    Soccer.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("1 Coin / 4 Credits", 0x06)
        .add("2 Coins / 1 Credit", 0x01)
        .add("3 Coins / 1 Credit", 0x05)
        .add("4 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    Soccer.add("Points Timer", 2)
        .add("600 Pts", 0x00)
        .add("800 Pts", 0x10)
        .add("1000 Pts", 0x08)
        .add("1200 Pts", 0x18);
    Soccer.add("Difficulty", 1)
        .add("Easy", 0x00)
        .add("Normal", 0x40)
        .add("Hard", 0x20)
        .add("Very Hard", 0x60);            
    
    StarLuster.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x02)
        .add("2 Coins / 1 Credit", 0x01)
        .add("3 Coins / 1 Credit", 0x03);
    StarLuster.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x04);
    StarLuster.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x08);
    StarLuster.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x10);
    StarLuster.add("Palette Color")
        .add("Black", 0x40)
        .add("Green", 0x20)
        .add("Grey", 0x60);
    StarLuster.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x80); 
    
    StrokeAndMatchGolf.addAll(LadiesGolf.getDipSwitches());

    StrokeAndMatchGolfJapan.addAll(LadiesGolf.getDipSwitches());
    
    SuperMarioBros.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x06)
        .add("1 Coin / 3 Credits", 0x01)
        .add("1 Coin / 4 Credits", 0x05)
        .add("1 Coin / 5 Credits", 0x03)
        .add("2 Coins / 1 Credit", 0x04)
        .add("3 Coins / 1 Credit", 0x02)
        .add("Free Play", 0x07);
    SuperMarioBros.add("Lives", 1)
        .add("2", 0x08)
        .add("3", 0x00);
    SuperMarioBros.add("Bonus Life")
        .add("100", 0x00)
        .add("150", 0x20)
        .add("200", 0x10)
        .add("250", 0x30);
    SuperMarioBros.add("Timer")
        .add("Normal", 0x00)
        .add("Fast", 0x40);
    SuperMarioBros.add("Continue Lives")
        .add("3", 0x80)
        .add("4", 0x00);    
    
    SuperSkyKid.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x01);
    SuperSkyKid.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x02);
    SuperSkyKid.add("Lives")
        .add("2", 0x00)
        .add("3", 0x04);
    SuperSkyKid.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x08)
        .add("2 Coins / 1 Credit", 0x10)
        .add("3 Coins / 1 Credit", 0x18);
    SuperSkyKid.add("PPU", 2)
        .add("RP2C03", 0x20)
        .add("RP2C04-0001", 0x00)
        .add("RP2C04-0002", 0x40)
        .add("RP2C04-0003", 0x80)
        .add("RP2C04-0004", 0xC0);
    
    SuperXevious.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x01);
    SuperXevious.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x02);
    SuperXevious.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x04);
    SuperXevious.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x08);
    SuperXevious.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x10)
        .add("2 Coins / 1 Credit", 0x20)
        .add("3 Coins / 1 Credit", 0x30);
    SuperXevious.add("PPU", 1)
        .add("RP2C04-0001", 0x00)
        .add("RP2C04-0002", 0x40)
        .add("RP2C04-0003", 0x80)
        .add("RP2C04-0004", 0xC0);
    
    Tetris.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x02)
        .add("2 Coins / 1 Credit", 0x01)
        .add("3 Coins / 1 Credit", 0x03);
    Tetris.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x04);
    Tetris.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x08);
    Tetris.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x10);
    Tetris.add("Palette Color")
        .add("Black", 0x40)
        .add("Green", 0x20)
        .add("Grey", 0x60);
    Tetris.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x80);    
    
    TkoBoxing.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x01)
        .add("2 Coins / 1 Credit", 0x02)
        .add("3 Coins / 1 Credit", 0x03);
    TkoBoxing.add("Unknown/Unused")
		    .add("Off", 0x00)
		    .add("On", 0x04);
    TkoBoxing.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x08);
    TkoBoxing.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x10);
    TkoBoxing.add("Palette Color", 1)
        .add("Black", 0x00)
        .add("White", 0x20);
    TkoBoxing.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x40);
    TkoBoxing.add("Unknown/Unused")
        .add("Off", 0x00)
        .add("On", 0x80); 
    
    TopGun.add("Coinage")
        .add("1 Coin / 1 Credit", 0x00)
        .add("1 Coin / 2 Credits", 0x04)
        .add("1 Coin / 3 Credits", 0x02)
        .add("2 Coins / 1 Credit", 0x06)
        .add("3 Coins / 1 Credit", 0x01)
        .add("4 Coins / 1 Credit", 0x05)
        .add("5 Coins / 1 Credit", 0x03)
        .add("Free Play", 0x07);
    TopGun.add("Lives per Coin")
        .add("3 - 12 Max", 0x00)
        .add("2 - 9 Max", 0x08);
    TopGun.add("Bonus")
        .add("30k and every 50k", 0x00)
        .add("50k and every 100k", 0x20)
        .add("100k and every 150k", 0x10)
        .add("200k and every 200k", 0x30);
    TopGun.add("Difficulty")
        .add("Normal", 0x00)
        .add("Hard", 0x40);
    TopGun.add("Demo Sounds", 1)
        .add("Off", 0x00)
        .add("On", 0x80);
  }
  
  private static final Object[][] NameMappings = {
    { "rbi", RbiBaseball },
    { "balloon", BalloonFight },
    { "baseball", Baseball },
    { "battle", BattleCity },
    { "land", CluCluLand },
    { "dr", DrMario },
    { "duck", DuckHunt },
    { "excite", Excitebike },
    { "force", FreedomForce },
    { "goonies", Goonies },
    { "gradius", Gradius },
    { "gumshoe", Gumshoe },
    { "alley", HogansAlley },
    { "climber", IceClimber },
    { "rider", MachRider },
    { "ladies", LadiesGolf },
    { "mahjong", Mahjong },
    { "mighty", MightyBombJack },
    { "ninja", NinjaJajamaruKun },
    { "pinball", Pinball },
    { "platoon", Platoon },
    { "soccer", Soccer },
    { "luster", StarLuster },
    { "stroke", StrokeAndMatchGolf },
    { "kid", SuperSkyKid },
    { "xevious", SuperXevious },
    { "tennis", Tennis },
    { "tetris", Tetris },
    { "top", TopGun },
    { "castle", Castlevania },
    { "slalom", Slalom },
    { "mario", SuperMarioBros },
    { "crew", WreckingCrew }, 
    { "tko", TkoBoxing },
  };
  
  private final List<DipSwitch> dipSwitches = new ArrayList<>();
  private final int mapper;
  private final int mirroring;
  private final int ppu;
  private final int mode;
  private final boolean swapControllers;
  private final boolean zapperGame;
  private final int ppuStatusID;
  
  VsGame(final int mapper, final int mirroring, final int ppu, final int mode,
      final boolean swapControllers, final boolean zapperGame) {
    this(mapper, mirroring, ppu, mode, swapControllers, zapperGame, 0);
  }

  VsGame(final int mapper, final int mirroring, final int ppu, final int mode,
      final boolean swapControllers, final boolean zapperGame,
          final int ppuStatusID) {    
    this.mapper = mapper;
    this.mirroring = mirroring;
    this.ppu = ppu;
    this.mode = mode;  
    this.swapControllers = swapControllers;
    this.zapperGame = zapperGame;
    this.ppuStatusID = ppuStatusID;
  }
  
  private void addAll(final List<DipSwitch> dipSwitches) {
    this.dipSwitches.addAll(dipSwitches);
  }
  
  private DipSwitch add(final String name) {
    return add(name, 0);
  }
  
  private DipSwitch add(final String name, final int defaultValue) {
    final DipSwitch dipSwitch = new DipSwitch(name, defaultValue);
    dipSwitches.add(dipSwitch);
    return dipSwitch;
  }

  public List<DipSwitch> getDipSwitches() {
    return dipSwitches;
  }

  public int getMapper() {
    return mapper;
  }
  
  public int getMirroring() {
    return mirroring;
  }  

  public int getPPU() {
    return ppu;
  }

  public int getMode() {
    return mode;
  }

  public boolean isSwapControllers() {
    return swapControllers;
  }

  public boolean isZapperGame() {
    return zapperGame;
  }

  public int getPpuStatusID() {
    return ppuStatusID;
  }
  
  public boolean isUniSystemGame() {
    return !isDualSystemGame();
  }
  
  public boolean isDualSystemGame() {
    return this == Baseball || this == BalloonFight || this == Mahjong 
        || this == Tennis || this == WreckingCrew;
  }
 
  public static VsGame getVsGame(final NesFile nesFile) {
    
    switch(nesFile.getFileCRC()) {
      case 0x672BD0E4:
      case 0xD699AE69:
        return BalloonFight;
      case 0x6750CA0F:
      case 0x2B511B51:
      case 0xFF84F552:
      case 0xD319144F:
      case 0x7712BD99:
      case 0x3B00B967:
      case 0xF11649E5:
        return Baseball;
      case 0xBFD87C08:
      case 0xB2816BF9:
      case 0x01357944:
        return BattleCity;
      case 0x0FA322C2:
        return CluCluLand;
      case 0x2B85420E:
        return DrMario;
      case 0xC1D6411F:
      case 0xABE1A0C2:
        return DuckHunt;      
      case 0xC95321A8:
        return Excitebike;
      case 0xFB0DDDE7:
        return ExcitebikeJapan;
      case 0x832CF592:
        return FreedomForce;
      case 0x5E35AD1D:
      case 0x0D9EEFFE:
      case 0xF6DE2AA2:
      case 0x74F713B4:
        return Goonies;
      case 0x3FBEFD71:
      case 0xF735D926:
        return Gradius;
      case 0x74C78E8C:
      case 0x7C7D861A:
      case 0xB194CA80:
        return Gumshoe;
      case 0x16AA4E2D:
        return HogansAlley;
      case 0xCBBEFD1F:
      case 0x32CD7DE2:
      case 0xEF7AF338:
        return IceClimber;      
      case 0xE98A8A4D:
      case 0xE8B20197:
        return MachRider;
      case 0x70C7D241:
      case 0x3F3B4285:
      case 0xFBA769F6:
      case 0xAEFFF79C:
        return Mahjong;
      case 0x535E6152:
        return MightyBombJack;
      case 0xC20E9CA7:
        return NinjaJajamaruKun;      
      case 0xCA3E9B1A:
        return Pinball;
      case 0x66471EFE:
        return PinballJapan;
      case 0x0A36E7CE:
      case 0x31678411:
      case 0x2019FE65:
        return Platoon;
      case 0x252B5E51:
      case 0xE45485A5:
        return RbiBaseball;        
      case 0x766C2CAC:        
        return Soccer;
      case 0x159EF3C1:
        return StarLuster;      
      case 0xB1C4C508:
      case 0x1DCE31E1:
      case 0xE85C74D8:
        return StrokeAndMatchGolf;
      case 0x9768E5E0:
        return StrokeAndMatchGolfJapan;
      case 0xDF07C203:
      case 0x89B56859:
      case 0x12012CD9:
        return SuperXevious;
      case 0xBB63540A:
      case 0x5DBEBD53:
      case 0x1C6BA3B2:
        return Tennis;
      case 0x6B6F2442:
      case 0xE9A6F17D:
        return Tetris;
      case 0x8C0C2DF5:
        return TopGun;
      case 0xBAB3DDB9:
      case 0xFFBEF374:
        return Castlevania;
      case 0x9AE2BAA0:
        return Slalom;
      case 0x7CFF0F84:
      case 0x1C7430F9:
      case 0x4B79ED83:
      case 0x4BF3972D:
      case 0x98E3C75A:
        return SuperMarioBros;
      case 0xCE1A8CC7:
      case 0xCF4E0082:
        return WreckingCrew;      
    }
    
    final String fileName = nesFile.getFileName();
    if (fileName.contains("(vs)")) {
      for(int i = 0; i < NameMappings.length; i++) {
        final Object[] mapping = NameMappings[i];
        if (fileName.contains((String)mapping[0])) {
          final VsGame game = (VsGame)mapping[1];
          switch(game) {
            case Excitebike:
              if (isJapan(fileName)) {
                return ExcitebikeJapan;
              }
              break;
            case MachRider:
              if (isJapan(fileName)) {
                return MachRiderJapan;
              }
              break;
            case Pinball:
              if (isJapan(fileName)) {
                return PinballJapan;
              }
              break;
            case StrokeAndMatchGolf:
              if (isJapan(fileName)) {
                return StrokeAndMatchGolfJapan;
              }
              break;
          }
          return game;
        }
      }
    }
    
    return null;
  }
  
  private static boolean isJapan(final String fileName) {
    return fileName.contains("(j)") || fileName.contains("(japan)")
        || fileName.contains("(as)") || fileName.contains("(asia)");
  }
}
