package nintaco.mappers.racermate;

import nintaco.files.*;
import nintaco.mappers.*;

// TODO IMPLEMENT RACERMATE BICYCLE INPUT

public class Racermate extends Mapper {
  
  private static final long serialVersionUID = 0;
  
  private int irqCounter;
  private int irqValue;
  
  public Racermate(final CartFile cartFile) {
    super(cartFile, 4, 2);
  }
  
  @Override
  protected int getChrRamSize(final CartFile cartFile) {
    return 0x10000;
  }
  
  @Override
  public void init() {
    setPrgBank(3, -1);
  }
  
  @Override
  public void resetting() {
    init();
  }

  @Override
  protected void writeRegister(final int address, final int value) {
    if (address < 0xC000) {
      writeBankSelect(value);
    } else {
      writeIrqAcknowledge(address, value);
    }
  }
  
  private void writeBankSelect(final int value) {
    setPrgBank(2, value >> 6);
    setChrBank(1, value & 0x0F);
  }
  
  private void writeIrqAcknowledge(final int address, final int value) {
    final int nextIrqValue = ((address & 0x0080) << 1) | value;
    if (nextIrqValue == 0 && irqValue > 0) {      
      irqCounter = 0;
      cpu.setMapperIrq(false);
    }
    irqValue = nextIrqValue;
  }

  @Override
  public void update() {    
    if (irqCounter++ == 1024) {
      cpu.setMapperIrq(true);                  
    }
  }
}
