package nintaco.preferences;

import nintaco.gui.spritesaver.SpriteSaverGamePrefs;
import nintaco.gui.ramwatch.RamWatchGamePrefs;
import nintaco.gui.mapmaker.MapMakerGamePrefs;
import nintaco.gui.dipswitches.DipSwitchesGamePrefs;
import nintaco.gui.debugger.DebuggerGamePrefs;
import nintaco.gui.debugger.logger.LoggerGamePrefs;
import nintaco.gui.api.local.ProgramGamePrefs;
import java.io.*;
import java.util.*;
import nintaco.gui.hexeditor.preferences.*;
import static nintaco.files.FileUtil.*;
import static nintaco.util.StreamUtil.*;
import static nintaco.util.ThreadUtil.*;

public final class GamePrefs implements Serializable {

  private static final long serialVersionUID = 0;
  
  private static final List<Thread> threads = Collections.synchronizedList(
      new ArrayList<>());
  
  private static GamePrefs instance;
  private static String fileName;
  
  private transient int[] nonVolatilePrgRam;
  private transient int[] turboFileRam;
  
  private HexEditorGamePrefs hexEditorGamePrefs;
  private MapMakerGamePrefs mapMakerGamePrefs;
  private SpriteSaverGamePrefs spriteSaverGamePrefs;
  private RamWatchGamePrefs ramWatchGamePrefs;
  private DebuggerGamePrefs debuggerGamePrefs;
  private LoggerGamePrefs loggerGamePrefs;
  private DipSwitchesGamePrefs dipSwitchesGamePrefs;
  private ProgramGamePrefs programGamePrefs;

  public int[] getNonVolatilePrgRam() {
    synchronized(GamePrefs.class) {
      return nonVolatilePrgRam;
    }
  }

  public void setNonVolatilePrgRam(final int[] nonVolatilePrgRam) {
    synchronized(GamePrefs.class) {
      this.nonVolatilePrgRam = nonVolatilePrgRam;
    }
  }

  public int[] getTurboFileRam() {
    synchronized(GamePrefs.class) {
      return turboFileRam;
    }
  }

  public void setTurboFileRam(final int[] turboFileRam) {
    synchronized(GamePrefs.class) {
      this.turboFileRam = turboFileRam;
    }
  }
  
  public LoggerGamePrefs getLoggerGamePrefs() {
    synchronized(GamePrefs.class) {
      if (loggerGamePrefs == null) {
        loggerGamePrefs = new LoggerGamePrefs();
      }
      return loggerGamePrefs;
    }
  }
  
  public DebuggerGamePrefs getDebuggerGamePrefs() {
    synchronized(GamePrefs.class) {
      if (debuggerGamePrefs == null) {
        debuggerGamePrefs = new DebuggerGamePrefs();
      }
      return debuggerGamePrefs;
    }
  }
  
  public RamWatchGamePrefs getRamWatchGamePrefs() {
    synchronized(GamePrefs.class) {
      if (ramWatchGamePrefs == null) {
        ramWatchGamePrefs = new RamWatchGamePrefs();
      }
      return ramWatchGamePrefs;
    }
  }  
  
  public SpriteSaverGamePrefs getSpriteSaverGamePrefs() {
    synchronized(GamePrefs.class) {
      if (spriteSaverGamePrefs == null) {
        spriteSaverGamePrefs = new SpriteSaverGamePrefs();
      }
      return spriteSaverGamePrefs;
    }
  }  
  
  public MapMakerGamePrefs getMapMakerGamePrefs() {
    synchronized(GamePrefs.class) {
      if (mapMakerGamePrefs == null) {
        mapMakerGamePrefs = new MapMakerGamePrefs();
      }
      return mapMakerGamePrefs;
    }
  }

  public HexEditorGamePrefs getHexEditorGamePrefs() {
    synchronized(GamePrefs.class) {
      if (hexEditorGamePrefs == null) {
        hexEditorGamePrefs = new HexEditorGamePrefs();
      }
      return hexEditorGamePrefs;
    }
  }
  
  public DipSwitchesGamePrefs getDipSwitchesGamePrefs() {
    synchronized(GamePrefs.class) {
      if (dipSwitchesGamePrefs == null) {
        dipSwitchesGamePrefs = new DipSwitchesGamePrefs();
      }
      return dipSwitchesGamePrefs;
    }
  }
  
  public ProgramGamePrefs getProgramGamePrefs() {
    synchronized(GamePrefs.class) {
      if (programGamePrefs == null) {
        programGamePrefs = new ProgramGamePrefs();
      }
      return programGamePrefs;
    }
  }

  private void readObject(ObjectInputStream in) throws IOException, 
      ClassNotFoundException {
    in.defaultReadObject();
    nonVolatilePrgRam = readSparseByteArray(in);
    turboFileRam = readSparseByteArray(in);
  }
  
  private void writeObject(ObjectOutputStream out) throws IOException {
    out.defaultWriteObject();
    writeSparseByteArray(out, nonVolatilePrgRam);
    writeSparseByteArray(out, turboFileRam);
  }  

  public static void dispose() {
    joinAll(threads);
    synchronized(GamePrefs.class) {
      instance = null;
    }
  }
  
  public synchronized static void load(final String entryFileName) {     
    fileName = createGamePreferencesFile(entryFileName);
    dispose();
    final File file = new File(fileName);
    if (file.exists()) {
      try (final ObjectInputStream in = new ObjectInputStream(
          new BufferedInputStream(new FileInputStream(fileName)))) {
        instance = (GamePrefs)in.readObject();
      } catch(final Throwable t) {
        //t.printStackTrace();
      }
    } 
    if (instance == null) {
      instance = new GamePrefs();
    }
  }
  
  public static void save() {
    if (fileName == null) {
      return;
    }
    final GamePrefs prefs = instance;
    if (prefs != null) {
      final Thread thread = new Thread(() -> {
        synchronized(GamePrefs.class) {
          mkdir(AppPrefs.getInstance().getPaths().getGamePreferencesDir());
          try(final ObjectOutputStream out = new ObjectOutputStream(
              new BufferedOutputStream(new FileOutputStream(fileName)))) {
            out.writeObject(prefs);
          } catch (Throwable t) {
            //t.printStackTrace();
          }
        }
        threads.remove(Thread.currentThread());
      }, "GamePrefs Save Thread");
      synchronized(threads) {
        thread.start();
        threads.add(thread);
      }
    }
  }
  
  public static GamePrefs getInstance() {    
    return instance;
  }
  
  private GamePrefs() {    
  }
}